# -*- coding: utf-8 -*-

import nose.tools as nt
from pyee import Event_emitter, EventEmitter


class ItWorkedException(Exception):
    """An exception, used to prove that an event handler fired.
    Kinda hack-y, I know.
    """
    pass

def test_emit():
    """Test that event_emitters fire properly.
    """
    ee = Event_emitter()

    # Used in a decorator style
    @ee.on('event')
    def event_handler(ev, data, **kwargs):
        nt.assert_equals(ev, 'event')
        nt.assert_equals(data, 'emitter is emitted!')
        # Raise exception to prove it fired.
        if (kwargs['error']):
            raise ItWorkedException

    # Making sure data is passed propers
    ee.emit('event', 'emitter is emitted!', error=False)

    # Some nose bullshit to check for the firings.
    # "Hides" other exceptions.
    with nt.assert_raises(ItWorkedException) as it_worked:
        ee.emit('event', 'emitter is emitted!', error=True)

def test_emit_error():
    ee = EventEmitter()

    with nt.assert_raises(Exception):
        ee.emit('error')

    @ee.on('error')
    def onError(ev):
        pass

    # No longer raises and error instead return True indicating handled
    nt.assert_true(ee.emit('error'))

def test_emit_return():
    ee = EventEmitter()

    # make sure emitting without a callback returns False
    nt.assert_false(ee.emit('data'))

    # add a callback
    ee.on('data')(lambda ev: None)

    # should return True now
    nt.assert_true(ee.emit('data'))

def test_new_listener_event():
    """Test the 'new_listener' event.
    """

    ee = Event_emitter()

    @ee.on('new_listener')
    def new_listener_handler(new_event, event, fxn):
        raise ItWorkedException

    with nt.assert_raises(ItWorkedException) as it_worked:
        @ee.on('event')
        def event_handler(ev, data):
            pass


def test_listener_removal():
    """Tests that we can remove listeners (as appropriate).
    """

    ee = Event_emitter()


    #Some functions to pass to the EE
    def first():
        return 1

    ee.on('event', first)

    @ee.on('event')
    def second():
        return 2

    @ee.on('event')
    def third():
        return 3

    def fourth():
        return 4

    ee.on('event', fourth)

    nt.assert_equal(ee._events['event'], [first, second, third, fourth])

    # uses the function itself to find/remove listener
    ee.remove_listener('event', second)
    nt.assert_equal(ee._events['event'], [first, third, fourth])

    # uses the function itself to find/remove listener
    ee.remove_listener('event', first)
    nt.assert_equal(ee._events['event'], [third, fourth])

    # Remove ALL listeners!
    ee.remove_all_listeners('event')
    nt.assert_equal(ee._events['event'], [])

def test_once():
    """Test that `once()` method works propers.
    """

    # very similar to "test_emit" but also makes sure that the event
    # gets removed afterwards

    ee = Event_emitter()

    def once_handler(ev, data, error=None):
        nt.assert_equals(data, 'emitter is emitted!')
        if (error):
            raise ItWorkedException

    #Tests to make sure that after event is emitted that it's gone.
    ee.once('event', once_handler)
    ee.emit('event', 'emitter is emitted!')
    nt.assert_equal(ee._events['event'], [])

    #Tests to make sure callback fires. "Hides" other exceptions.
    with nt.assert_raises(ItWorkedException) as it_worked:
        ee.once('event', once_handler)
        ee.emit('event', 'emitter is emitted!', True)

def test_listeners():
    """Test that `listeners()` gives you access to the listeners array.
    """

    ee = Event_emitter()
    @ee.on('event')
    def event_handler():
        pass

    def raiser(ev):
        raise ItWorkedException

    l = ee.listeners('event')
    l[0] = raiser

    with nt.assert_raises(ItWorkedException) as it_worked:
        ee.emit('event')



# -*- coding: utf-8 -*-

import nose.tools as nt
from pyee import Event_emitter, EventEmitter, PatternException
from tests import ItWorkedException


def test_is_pattern():
    ee = EventEmitter()
    assert ee._isPattern('a/#')
    assert ee._isPattern('a/+/+')
    assert not ee._isPattern('a/#/+')
    assert not ee._isPattern('a/c+/c')
    assert not ee._isPattern('a/#/c')


def test_pattern_matching():
    """Test that patterns are correctly interpreted"""
    ee = EventEmitter()
    assert ee._matches('#', 'a/b/c')
    assert ee._matches('+/b/c', 'a/b/c')
    assert ee._matches('a/#', 'a/b/c')
    assert not ee._matches('a/#', 'c/a/b/c')
    with nt.assert_raises(PatternException) as e:
        ee._matches('#/b/c', 'c')
    assert not ee._matches('a/+/d/e', 'a/b/c/d/e')


def test_matching_topic():
    """Test that a pattern can be passed as an event"""

    ee = Event_emitter()

    @ee.on('event/+/ok')
    def event_handler(ev):
        raise ItWorkedException

    with nt.assert_raises(ItWorkedException) as it_worked:
        ee.emit('event/first/ok')

    with nt.assert_raises(ItWorkedException) as it_worked:
        ee.emit('event/second/ok')

    ee.emit('event/first/ok2')


def test_shorter_pattern():
    """Tests correct behaviour with shorter patterns"""

    ee = EventEmitter()

    @ee.on('#')
    def event_handler(ev):
        raise ItWorkedException

    with nt.assert_raises(ItWorkedException) as it_worked:
        ee.emit('a/b/c')

    with nt.assert_raises(ItWorkedException) as it_worked:
        ee.emit('cool')


def test_longer_pattern():
    """Tests correct behaviour with longer patterns"""

    ee = EventEmitter()

    @ee.on('a/b/#')
    def event_handler(ev):
        raise ItWorkedException

    ee.emit('c')

    @ee.on('+/a/b')
    def event_handler(ev):
        raise ItWorkedException('c and #/a/b')

    ee.emit('c')



# -*- coding: utf-8 -*-

"""
pyee
====

pyee supplies an ``EventEmitter`` object similar to the ``EventEmitter``
from Node.js.

There is also the possibility to use mqtt topic patterns to match events

Example
-------

::

    In [1]: from pyee import EventEmitter

    In [2]: ee = EventEmitter()

    In [3]: @ee.on('event')
       ...: def event_handler():
       ...:     print 'BANG BANG'
       ...:

    In [4]: ee.emit('event')
    BANG BANG

    In [5]:

    In [6]: ee.on('a/#/c', lambda ...)

Easy-peasy.


"""


class PatternException(Exception):
    pass

from collections import defaultdict

__all__ = ['EventEmitter', 'Event_emitter']


class EventEmitter(object):

    """The EventEmitter class.

    (Special) Events
    ----------------

    -   'new_listener': Fires whenever a new listener is created. Listeners for this
    event do not fire upon their own creation.

    -   'error': When emitted raises an Exception by default, behavior can be overriden by
    attaching callback to the event.

    For example::

        @ee.on('error')
        def onError(message):
            logging.err(message)

        ee.emit('error', Exception('something blew up'))

    """

    def __init__(self):
        """
        Initializes the EE.
        """
        self._events = defaultdict(list)

        # Specialised dict for pattern matching topics
        # This way it will not impact standard behaviour
        self._patterns = defaultdict(list)

    def on(self, event, f=None):
        """Registers the function ``f`` to the event name ``event``.

        If ``f`` isn't provided, this method returns a function that
        takes ``f`` as a callback; in other words, you can use this method
        as a decorator, like so:

            @ee.on('data')
            def data_handler(data):
                print data

        """

        def _on(f):
            # Fire 'new_listener' *before* adding the new listener!
            self.emit('new_listener', event, f)

            # Add the necessary function
            if self._isPattern(event):
                self._patterns[event].append(f)
            else:
                self._events[event].append(f)

            # Return original function so removal works
            return f

        if f is None:
            return _on
        else:
            return _on(f)

    def emit(self, event, *args, **kwargs):
        """Emit ``event``, passing ``*args`` to each attached function. Returns
        ``True`` if any functions are attached to ``event``; otherwise returns
        ``False``.

        Example:

            ee.emit('data', '00101001')

        Assuming ``data`` is an attached function, this will call
        ``data('data', '00101001')'``.

        """
        handled = False

        for p in self._patterns:
            if self._matches(p, event):
                for f in self._patterns[p]:
                    f(event, *args, **kwargs)
                    handled = True

        # Pass the args to each function in the events dict
        for f in self._events[event]:
            f(event, *args, **kwargs)
            handled = True

        if not handled and event == 'error':
            raise Exception("Uncaught 'error' event.")

        return handled

    def _isPattern(self, pattern):
        """is there any /+/ or /#/ in the pattern?"""
        if '#' in pattern and not pattern.endswith('#'):
            return False
        return any(filter(lambda x: x == '+' or x == '#', pattern.split('/')))

    def _matches(self, pattern, event):
        "Check that 'a/+/b/+/c' pattern matches '/a/x/b/x/c'"

        if '#' in pattern and not pattern.endswith('#'):
            raise PatternException

        ps, es = pattern.split('/'), event.split('/')
        return all([x == y or x == '+' or (x == '#' and len(ps) <= len(es)) for x, y in zip(ps, es)]) and len(ps) <= len(es)

    def once(self, event, f=None):
        """The same as ``ee.on``, except that the listener is automatically
        removed after being called.
        """
        def _once(f):
            def g(*args, **kwargs):
                f(*args, **kwargs)
                self.remove_listener(event, g)
            return g

        if f is None:
            return lambda f: self.on(event, _once(f))
        else:
            self.on(event, _once(f))

    def remove_listener(self, event, f):
        """Removes the function ``f`` from ``event``.

        Requires that ``f`` is not closed over by ``ee.on``. (In other words,
        it is, unfortunately, not possible to use this with the decorator
        style is.)

        """
        if self._isPattern(event):
            self._patterns[event].remove(f)
        else:
            self._events[event].remove(f)

    def remove_all_listeners(self, event=None):
        """Remove all listeners attached to ``event``.
        """
        if event is not None:
            if '#' in event:
                self._patterns[event] = []
            else:
                self._events[event] = []
        else:
            self._events = None
            self._patterns = None
            self._events = defaultdict(list)
            self._patterns = defaultdict(list)

    def listeners(self, event):
        """Returns the list of all listeners registered to the ``event``.
        """
        return self._events[event] if not self._isPattern(event) else self._patterns[event]

# Backwards compatiblity
Event_emitter = EventEmitter



# -*- coding: utf-8 -*-

import os
from distutils.core import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "pyee-topics",
    version = "0.0.13",
    packages = ["pyee"],
    description = "A port of node.js's EventEmitter to python with mqtt topics support.",
    author = "Joseph Piron (Joshua Holbrook)",
    author_email = "joseph.piron@gmail.com (josh.holbrook@gmail.com)",
    url = "https://github.com/eagleamon/pyee-topics",
    keywords = ["events", "emitter", "node.js", "node", "eventemitter", "event_emitter", "mqtt", "patterns", "topics"],
    classifiers = [
        "Programming Language :: Python",
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7", #only one tested
        "Topic :: Other/Nonlisted Topic"
    ],
    long_description = read('README.rst')
)
